@@ -12,6 +12,7 @@ from packaging.version import parse as parse_version
 import dask
 from dask.base import tokenize
 from dask.blockwise import BlockIndex
+from dask.dataframe.backends import dataframe_creation_dispatch
 from dask.dataframe.core import DataFrame, Scalar
 from dask.dataframe.io.io import from_map
 from dask.dataframe.io.parquet.utils import Engine, _sort_and_analyze_paths
@@ -171,6 +172,7 @@ class ToParquetFunctionWrapper:
         )
 
 
+@dataframe_creation_dispatch.register_inplace("pandas")
 def read_parquet(
     path,
     columns=None,
@@ -221,7 +223,7 @@ def read_parquet(
         of the combined filters with an OR disjunction.
 
         Predicates can also be expressed as a ``List[Tuple]``. These are evaluated
-        as an AND conjunction. To express OR in predictates, one must use the
+        as an AND conjunction. To express OR in predicates, one must use the
         (preferred for "pyarrow") ``List[List[Tuple]]`` notation.
 
         Note that the "fastparquet" engine does not currently support DNF for
@@ -257,7 +259,9 @@ def read_parquet(
         This option will be ignored if ``index`` is not specified and there is
         no physical index column specified in the custom "pandas" Parquet
         metadata. Note that ``calculate_divisions=True`` may be extremely slow
-        on some systems, and should be avoided when reading from remote storage.
+        when no global ``_metadata`` file is present, especially when reading
+        from remote storage. Set this to ``True`` only when known divisions
+        are needed for your workload (see :ref:`dataframe-design-partitions).
     ignore_metadata_file : bool, default False
         Whether to ignore the global ``_metadata`` file (when one is present).
         If ``True``, or if the global ``_metadata`` file is missing, the parquet
@@ -1241,7 +1245,7 @@ def apply_filters(parts, statistics, filters):
         filters with an OR disjunction.
 
         Predicates can also be expressed as a List[Tuple]. These are evaluated
-        as an AND conjunction. To express OR in predictates, one must use the
+        as an AND conjunction. To express OR in predicates, one must use the
         (preferred) List[List[Tuple]] notation.
 
         Note that the "fastparquet" engine does not currently support DNF for
